LANGUAGE1 = "Dutch"
LANGUAGE2 = "English"
CODE_L1 = "nl"
CODE_L2 = "en"

FILE_ENCODING = 'utf-8'

#GUI

COLOR_BKG_INACTIVE = "#8A8A8A"
COLOR_BKG_ACTIVE = "#FFFFFF"
COLOR_BKG_ACTIVE_INFO = "#f6f6f6"
COLOR_BKG_TAX_LIST = COLOR_BKG_ACTIVE_INFO
COLOR_QUEST = "#BBBBDD"
COLOR_SOLUTION = "#BBDDBB"
COLOR_RED_BTN = ("#000000", "#DDBBBB")
COLOR_GREEN_BTN = ("#000000", "#BBDDBB")

SG_COLOR_THEME = {'BACKGROUND': '#E0E0E2',
                  'TEXT': '#000000',
                  'INPUT': COLOR_BKG_ACTIVE_INFO,
                  'TEXT_INPUT': '#000000',
                  'SCROLL': '#99CC99',
                  'BUTTON': ('#000000', '#d4d7dd'),
                  'PROGRESS': ('#D1826B', '#CC8019'),
                  'BORDER': 1, 'SLIDER_DEPTH': 0,
                  'PROGRESS_DEPTH': 0, }

WIDTH_ML = 80 # multi line field for text input
LEN_ML_SMALL = 6
LEN_ML_LARGE = 15 # tab layout
LEN_ANSWER_SMALL = 5
LEN_ANSWER_LARGE = 8
TAB_LAYOUT = True

MAX_RECENT_DIRS = 5