UNKNOWN_TYPE = "unknown"
EXTYPES = {"schoice": "Multiple choice",
           "mchoice": "Multiple answer",
           "num": "Fill in the blank numbers",
           "string": "Fill in the blank text/essay",
           "cloze": "Combinations"}

HAVE_ANSWER_LIST = ["schoice", "mchoice", "cloze"]
